<?php
  /* ***** BEGIN LICENSE BLOCK *****
   * Version: MPL 1.1/GPL 2.0/LGPL 2.1
   *
   * The contents of this file are subject to the Mozilla Public License
   * Version 1.1 (the "MPL"); you may not use this file except in
   * compliance with the MPL. You may obtain a copy of the MPL at
   * http://www.mozilla.org/MPL/
   *
   * Software distributed under the MPL is distributed on an "AS IS" basis,
   * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
   * for the specific language governing rights and limitations under the
   * MPL.
   *
   * The Original Code is Xplico Interface (XI).
   *
   * The Initial Developer of the Original Code is
   * Gianluca Costa <g.costa@xplico.org>
   * Portions created by the Initial Developer are Copyright (C) 2007
   * the Initial Developer. All Rights Reserved.
   *
   * Contributor(s):
   *
   * Alternatively, the contents of this file may be used under the terms of
   * either the GNU General Public License Version 2 or later (the "GPL"), or
   * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   * in which case the provisions of the GPL or the LGPL are applicable instead
   * of those above. If you wish to allow use of your version of this file only
   * under the terms of either the GPL or the LGPL, and not to allow others to
   * use your version of this file under the terms of the MPL, indicate your
   * decision by deleting the provisions above and replace them with the notice
   * and other provisions required by the GPL or the LGPL. If you do not delete
   * the provisions above, a recipient may use your version of this file under
   * the terms of any one of the MPL, the GPL or the LGPL.
   *
   * ***** END LICENSE BLOCK ***** */

uses ('sanitize');

class SolsController extends AppController {
    var $name = 'Sols';
    var $helpers = array('Html', 'Form', 'Javascript');
    var $components = array('Xplico');
    var $uses = array('Sol', 'Pol', 'Source', 'Email', 'Web', 'Ftp', 'Ftp_file', 'Sip', 'Mm', 'Mmscontent', 'Pjl', 'Feed', 'Tftp', 'Tftp_file', 'DnsMessage', 'Nntp_group', 'Nntp_article', 'Fbuchat', 'Fbchat', 'Telnet', 'Webmail', 'Httpfile', 'Unknow', 'Rtp', 'Arp', 'Irc', 'Irc_channel', 'Paltalk_exp', 'Paltalk_room', 'Msn_chat');
    function beforeFilter() {
        $groupid = $this->Session->read('group');
        $polid = $this->Session->read('pol');
        if (!$groupid || !$polid) {
            $this->Session->delete('sol');
            $this->Session->delete('host_id');
            $this->redirect('/users/login');
        }
    }
    
    function index() {
        $polid = $this->Session->read('pol');
        $this->Session->delete('host_id');
        $this->Sol->recursive = -1;
        $this->Pol->recursive = -1;
        $pol_data = $this->Pol->read(null, $polid);
        $sols = $this->Sol->findAll("pol_id = $polid", NULL, 'Sol.id DESC');
        $this->set('pol_name', $pol_data['Pol']['name']);
        $this->set('sols', $sols);
        if ($this->Session->check('admin')) {
            // admin menu
            $this->set('menu_left', 
                   array('active' => '0', 'sections' => array(
                             array('name' => 'Case', 'sub' => array(
                                       array('name' => 'Cases', 'link' => '/pols'),
                                       array('name' => 'Sessions', 'link' => '/sols/index')
                                       )
                                 )
                             )
                       )
            );
            
        }
        else {
            // normal user menu
            $this->set('menu_left', 
                   array('active' => '0', 'sections' => array(
                             array('name' => 'Case', 'sub' => array(
                                       array('name' => 'Cases', 'link' => '/pols'),
                                       array('name' => 'Sessions', 'link' => '/sols/index'),
                                       array('name' => 'New Session', 'link' => '/sols/add')
                                       )
                                 )
                             )
                       )
            );
        }
        if (!empty($sols['0']['Sol']['id']))
            $this->Session->write('last_sol_id', $sols['0']['Sol']['id']);
    }
    
    function view($id = null) {
        if (!$id) {
            $this->Session->setFlash('Invalid id for Session.');
            $this->redirect('/sols/index');
        }
        $this->Session->delete('interface');
        $this->Sol->recursive = 0;
        $sol = $this->Sol->read(null, $id);
        if ($sol['Sol']['pol_id'] == $this->Session->read('pol')) {
            $this->Session->write('sol', $id);
            $live = $sol['Pol']['realtime'];
            $this->set('live', $live);
            if ($live) {
                $pol = $this->Session->read('pol');
                $start_file = '/opt/xplico/pol_'.$pol.'/realtime_start';
                if (file_exists($start_file)) {
                    $this->set('livestop', 1);
                    
                    /* SEND THE INTERFACE IN USE TO THE VIEW ALND DISPLAY IT */
                    $this->set('interff', $this->Session->read('interfaceInUse'));
                    
                }
                else {
                    $interface = array();
                    $foca = popen('/sbin/ifconfig -a | grep "Link encap" | cut -b -9', 'r');
                    if ($foca) {
                        while (!feof($foca)) {
                            $buffer = trim(fgets($foca, 200));
                            if ($buffer != '') {
                                        $interface[] = $buffer;
                            }
                        }
                        pclose($foca);
                    }
                    $this->set('interface', $interface);
                    $this->Session->write('interface', $interface);
                    $this->set('livestop', 0);
                }
            }
            // selected host
            $host_id = $this->Session->read('host_id');
            $host_srch = "";
            if (!empty($host_id) && $host_id != 0) {
                $host_srch = " AND source_id = ".$host_id;
                $this->set('host', $host_id);
            }
            else {
                $this->set('host', 0);
            }
            // hosts list
            $this->Source->recursive = -1;
            $conditions = array(     
                'conditions' => array('Source.sol_id' => $id),
                'order' => 'ip ASC', 
                'limit' => null,
                'fields' => array(str_replace('{n}.','', "{n}.Source.id"), str_replace('{n}.','',"{n}.Source.ip"))    );
            $this->set('hosts',  $this->Source->find("list", $conditions));
            
            // email number
            $this->Email->recursive = -1;
            $eml_received =  $this->Email->findCount("sol_id = $id AND receive = 1".$host_srch, -1);
            $eml_total = $this->Email->findCount("sol_id = $id".$host_srch, -1);
            $eml_sended = $eml_total - $eml_received;
            $eml_unread =  $eml_total - $this->Email->findCount("sol_id = $id AND first_visualization_user_id != 0".$host_srch, -1);
            $this->set('eml_received', $eml_received);
            $this->set('eml_sended',   $eml_sended);
            $this->set('eml_unread',   $eml_unread);
            $this->set('eml_total',    $eml_total);
            // web number
            $this->Web->recursive = -1;
            $web_post = $this->Web->findCount("sol_id = $id AND method = 'POST'".$host_srch, -1);
            $web_get = $this->Web->findCount("sol_id = $id AND method = 'GET'".$host_srch, -1);
            $web_video = $this->Web->findCount("sol_id = $id AND content_type LIKE '%video%'".$host_srch, -1);
            $web_image = $this->Web->findCount("sol_id = $id AND content_type LIKE '%image%'".$host_srch, -1);
            $this->set('web_post', $web_post);
            $this->set('web_get', $web_get);
            $this->set('web_video', $web_video);
            $this->set('web_image', $web_image);
            // ftp number
            $this->Ftp->recursive = -1;
            $this->Ftp_file->recursive = -1;
            $ftp_num = $this->Ftp->findCount("sol_id = $id".$host_srch, -1);
            $ftp_down = $this->Ftp_file->findCount("sol_id = $id AND dowloaded = 1".$host_srch, -1);
            $ftp_up = $this->Ftp_file->findCount("sol_id = $id".$host_srch, -1) - $ftp_down;
            $this->set('ftp_num', $ftp_num);
            $this->set('ftp_down', $ftp_down);
            $this->set('ftp_up', $ftp_up);
            // mms number
            $this->Mm->recursive = -1;
            $this->Mmscontent->recursive = -1;
            $mms_num = $this->Mm->findCount("sol_id = $id".$host_srch, -1);
            $mms_cont = $this->Mmscontent->findCount("sol_id = $id".$host_srch, -1) - $mms_num;
            $mms_video = $this->Mmscontent->findCount("sol_id = $id AND content_type LIKE '%video%'".$host_srch, -1);
            $mms_image = $this->Mmscontent->findCount("sol_id = $id AND content_type LIKE '%image%'".$host_srch, -1);
            $this->set('mms_num', $mms_num);
            $this->set('mms_cont', $mms_cont);
            $this->set('mms_video', $mms_video);
            $this->set('mms_image', $mms_image);
            // pjl number
            $this->Pjl->recursive = -1;
            $pjl_num = $this->Pjl->findCount("sol_id = $id".$host_srch, -1);
            $this->set('pjl_num', $pjl_num);
            // feed number
            $this->Feed->recursive = -1;
            $feed_num = $this->Feed->findCount("sol_id = $id".$host_srch, -1);
            $this->set('feed_num', $feed_num);                    
            // tftp number
            $this->Tftp->recursive = -1;
            $this->Tftp_file->recursive = -1;
            $tftp_num = $this->Tftp->findCount("sol_id = $id".$host_srch, -1);
            $tftp_down = $this->Tftp_file->findCount("sol_id = $id AND dowloaded = 1".$host_srch, -1);
            $tftp_up = $this->Tftp_file->findCount("sol_id = $id".$host_srch, -1) - $tftp_down;
            $this->set('tftp_num', $tftp_num);
            $this->set('tftp_down', $tftp_down);
            $this->set('tftp_up', $tftp_up);
            // sip number
            $this->Sip->recursive = -1;
            $sip_calls = $this->Sip->findCount("sol_id = $id".$host_srch, -1);
            $this->set('sip_calls', $sip_calls);
            // rtp number
            $this->set('rtp_video', 0);
            $this->Rtp->recursive = -1;
            $rtp_audio = $this->Rtp->findCount("sol_id = $id".$host_srch, -1);
            $this->set('rtp_audio', $rtp_audio);
            // dns number
            $this->DnsMessage->recursive = -1;
            $dns_num = $this->DnsMessage->findCount("sol_id = $id".$host_srch, -1);
            $this->set('dns_num', $dns_num);
            // nntp number
            $this->Nntp_group->recursive = -1;
            $nntp_grp = $this->Nntp_group->findCount("sol_id = $id".$host_srch, -1);
            $this->set('nntp_grp', $nntp_grp);
            $this->Nntp_article->recursive = -1;
            $nntp_artcl = $this->Nntp_article->findCount("sol_id = $id".$host_srch, -1);
            $this->set('nntp_artcl', $nntp_artcl);
            // facebook chat number
            $this->Fbuchat->recursive = -1;
            $fbc_users = $this->Fbuchat->findCount("sol_id = $id".$host_srch, -1);
            $this->set('fbc_users', $fbc_users);
            $this->Fbchat->recursive = -1;
            $fbc_chats = $this->Fbchat->findCount("sol_id = $id".$host_srch, -1);
            $this->set('fbc_chats', $fbc_chats);
            // telnet number
            $this->Telnet->recursive = -1;
            $telnet_num = $this->Telnet->findCount("sol_id = $id".$host_srch, -1);
            $this->set('telnet_num', $telnet_num);
            // webmail
            $this->Webmail->recursive = -1;
            $webmail_num = $this->Webmail->findCount("sol_id = $id".$host_srch, -1);
            $webmail_received =  $this->Webmail->findCount("sol_id = $id AND receive = 1".$host_srch, -1);
            $this->set('webmail_num', $webmail_num);
            $this->set('webmail_receiv', $webmail_received);
            $this->set('webmail_sent', $webmail_num - $webmail_received);
            // httpfile number
            $this->Http->recursive = -1;
            $httpfile_num = $this->Httpfile->findCount("sol_id = $id".$host_srch, -1);
            $this->set('httpfile_num', $httpfile_num);
            // unknow number
            $this->Unknow->recursive = -1;
            $text_num = $this->Unknow->findCount("sol_id = $id AND Unknow.file_path != 'None'".$host_srch, -1);
            $unk_num = $this->Unknow->findCount("sol_id = $id".$host_srch, -1);
            $this->set('text_num', $text_num.'/'.$unk_num);
            // arp number
            $this->Arp->recursive = -1;
            $arp_num = $this->Arp->findCount("sol_id = $id", -1);
            $this->set('arp_num', $arp_num);
            // irc number
            $this->Irc->recursive = -1;
            $irc_num = $this->Irc->findCount("sol_id = $id".$host_srch, -1);
            $this->set('irc_num', $irc_num);
            // irc channels number
            $this->Irc_channel->recursive = -1;
            $irc_chnl_num = $this->Irc_channel->findCount("sol_id = $id".$host_srch, -1);
            $this->set('irc_chnl_num', $irc_chnl_num);
            // Paltalk Express number
            $this->Paltalk_exp->recursive = -1;
            $paltalk_exp_num = $this->Paltalk_exp->findCount("sol_id = $id".$host_srch, -1);
            $this->set('paltalk_exp_num', $paltalk_exp_num);
            // Paltalk
            $this->Paltalk_room->recursive = -1;
            $paltalk_num = $this->Paltalk_room->findCount("sol_id = $id".$host_srch, -1);
            $this->set('paltalk_num', $paltalk_num);
            // msn
            $this->Msn_chat->recursive = -1;
            $msn_num = $this->Msn_chat->findCount("sol_id = $id".$host_srch, -1);
            $this->set('msn_num', $msn_num);
            
            // menu
            $last_sol_id = $this->Session->read('last_sol_id');
            if ($last_sol_id == $id && !$this->Session->check('admin'))
                $this->set('last_sol', 1);
            else
                $this->set('last_sol', 0);
            $this->set('sol', $sol);
            
            $this->set('menu_left', $this->Xplico->leftmenuarray(0) );
            
            $this->set('refresh_time', 30);
        }
        else {
            $this->redirect('/pols/index');
        }
    }
    
    function add() {
        if ($this->Session->check('admin')) {
            $this->Session->setFlash('Administrators can not create new Cases or new Sessions!');
            $this->redirect('/sols/index');
        }
        $this->set('menu_left', 
                   array('active' => '0', 'sections' => array(
                             array('name' => 'Case', 'sub' => array(
                                       array('name' => 'Cases', 'link' => '/pols'),
                                       array('name' => 'Sessions', 'link' => '/sols/index')
                                           )
                                 )
                             )
                       )
            );
        if (empty($this->data)) {
            $pol = $this->Session->read('pol');
            $start_file = '/opt/xplico/pol_'.$pol.'/realtime_start';
            if (file_exists($start_file)) {
                $this->Session->setFlash('Before add a new session stop the live acquisition!');
                $this->redirect('/sols/index');
            }
            else {
                $this->render();
            }
        }
        else {
            $polid = $this->Session->read('pol');
            $this->data['Sol']['pol_id'] = $polid;
            if($this->Sol->save(Sanitize::paranoid($this->data))) {
                system('cd /opt/xplico; /opt/xplico/script/session_mng.pyc -s '. $polid . ' ' . $this->Sol->getLastInsertId());
                $this->Session->setFlash('The Session has been created');
                $this->redirect('/sols/index');
            }
            else {
                $this->Session->setFlash('Please correct errors below.');
            }
        }
    }
    
    function delete($id = null) {
        if (!$id) {
            $this->Session->setFlash('Invalid id for Session');
            $this->redirect('/sols/index');
        }
        $this->Sol->recursive = 0;
        $sol = $this->Sol->read(null, $id);
        if ($sol['Sol']['pol_id'] != $this->Session->read('pol')) {
            $this->Session->setFlash('Invalid id for Session');
            $this->redirect('/sols/index');
        }
        // request deletion to dema
        $sol['Sol']['rm'] = 1;
        $this->Sol->save($sol);
        // delete directory
        $sol_dir = '/opt/xplico/pol_'.$this->Session->read('pol').'/sol_'.$id;
        system('mv '.$sol_dir.'  '.$sol_dir.'rm');
        do {
            sleep(1);
        } while(file_exists($sol_dir.'rm'));
        $this->Session->setFlash('Session deleted');
        $this->redirect('/sols/index');
    }
    
    function pcap() {
        if (!empty($this->data) &&
            is_uploaded_file($this->data['Sols']['File']['tmp_name'])) {
            $filedec = '/opt/xplico/pol_'.$this->Session->read('pol').'/sol_'.$this->Session->read('sol').'/new/'.$this->data['Sols']['File']['name'];
            move_uploaded_file($this->data['Sols']['File']['tmp_name'], $filedec);
            $this->Session->setFlash('File uploaded, wait start decoding...');
            $this->redirect('/sols/view/'.$this->Session->read('sol'));
        }
        else {
            $this->Session->setFlash('Upload failed, please check <u><a href="http://forum.xplico.org/viewtopic.php?f=3&t=167"> this</a></u>.');
            $this->redirect('/sols/index');
        }
    }
    
    function live() {
        $interface = $this->Session->read('interface');
        $this->Session->delete('interface'); /*to-do claro, aquí me hace un delete */
        $sol = $this->Session->read('sol');
        $pol = $this->Session->read('pol');
        
        if (!empty($this->data)) {
            if ($interface[$this->data['Interface']['Type']] != '') {
                
                $this->Session->write('interfaceInUse', $interface[$this->data['Interface']['Type']]); 
                
                $start_file = '/opt/xplico/pol_'.$pol.'/realtime_start';
                $fp = fopen($start_file, 'w');
                fwrite($fp, $interface[$this->data['Interface']['Type']]."\n");
                fwrite($fp, 'not ((host '.$_SERVER['SERVER_ADDR'].') && (tcp port '.$_SERVER['SERVER_PORT']."))\n");
                fclose($fp);
                // update time
                $sold = $this->Sol->read(null, $sol);
                if ($sold['Sol']['start_time'] == '0000-00-00 00:00:00') {
                    $sold['Sol']['start_time'] = date('Y-m-d H:i:s');
                    $this->Sol->save($sold);
                }
                $this->Session->setFlash('Live capture started.');
            }
            else {
                $this->Session->setFlash('Select an Interface!');
            }
            $this->redirect('/sols/view/'.$sol);
        }
        else {
            $this->Session->setFlash('Start Failed');
            $this->redirect('/sols/index');
        }
        
    }
    
    function livestop() {
        $sol = $this->Session->read('sol');
        $pol = $this->Session->read('pol');
        $start_file = '/opt/xplico/pol_'.$pol.'/realtime_start';
        if (file_exists($start_file)) {
            $start_file = '/opt/xplico/pol_'.$pol.'/realtime_stop';
            $fp = fopen($start_file, 'w');
            // wait stop
            while ($fp) {
                fclose($fp);
                sleep(1);
                $fp = fopen($start_file, 'r');
            }
            // update time
            $sold = $this->Sol->read(null, $sol);
            $sold['Sol']['end_time'] = date('Y-m-d H:i:s');
            $this->Sol->save($sold);
            $this->Session->setFlash('Live capture stoped.');
        }
        $this->redirect('/sols/view/'.$sol);
    }
    
    function host() {
        $this->Session->write('host_id', $this->data['host']);
        $sol = $this->Session->read('sol');
        $this->redirect('/sols/view/'.$sol);
    }
}
?>
