<?php
  /* ***** BEGIN LICENSE BLOCK *****
   * Version: MPL 1.1/GPL 2.0/LGPL 2.1
   *
   * The contents of this file are subject to the Mozilla Public License
   * Version 1.1 (the "MPL"); you may not use this file except in
   * compliance with the MPL. You may obtain a copy of the MPL at
   * http://www.mozilla.org/MPL/
   *
   * Software distributed under the MPL is distributed on an "AS IS" basis,
   * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
   * for the specific language governing rights and limitations under the
   * MPL.
   *
   * The Original Code is Xplico Interface (XI).
   *
   * The Initial Developer of the Original Code is
   * Gianluca Costa <g.costa@xplico.org>
   * Portions created by the Initial Developer are Copyright (C) 2007
   * the Initial Developer. All Rights Reserved.
   *
   * Contributor(s):
   *
   * Alternatively, the contents of this file may be used under the terms of
   * either the GNU General Public License Version 2 or later (the "GPL"), or
   * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   * in which case the provisions of the GPL or the LGPL are applicable instead
   * of those above. If you wish to allow use of your version of this file only
   * under the terms of either the GPL or the LGPL, and not to allow others to
   * use your version of this file under the terms of the MPL, indicate your
   * decision by deleting the provisions above and replace them with the notice
   * and other provisions required by the GPL or the LGPL. If you do not delete
   * the provisions above, a recipient may use your version of this file under
   * the terms of any one of the MPL, the GPL or the LGPL.
   *
   * ***** END LICENSE BLOCK ***** */

uses('sanitize');
class UsersController extends AppController
{
    var $helpers = array('Html', 'Form', 'Javascript', 'Ajax' );
    var $components = array('RequestHandler', 'Security', 'Xplico');
    var $uses = array('Param', 'User', 'Group');
    
    function beforeFilter() {
        // data input
        if (!empty($this->data)) {
            $this->Security->requirePost('register');
            $this->Security->requirePost('login');
        }
        $this->Security->blackHoleCallback='invalid';

        // dema running
    }

    function invalid() {
        header('HTTP/x 400 Bad Request');
        echo('<h1>HTTP: 400 Bad Request</h1>');
        echo('<p>We\'re sorry - there has been a problem processing your request.  Please try submitting the form again.</p>');
        die();
    }

    function about() {
    }

    function licenses() {
    }

    function cc_by_nc_sa() {
        $this->layout = 'licenses';
    }

    function gpl() {
        $this->layout = 'licenses';
    }    

    function help() {
        $solid = $this->Session->read('sol');
        if (isset($solid)) {
            $this->set('menu_left', $this->Xplico->leftmenuarray(0));
        }
        else {
            $polid = $this->Session->read('pol');
            if (isset($polid)) {
                $this->set('menu_left', 
                           array('active' => '0', 'sections' => array(
                                     array('name' => 'Case', 'sub' => array(
                                               array('name' => 'Cases', 'link' => '/pols'),
                                               array('name' => 'Sessions', 'link' => '/sols/index')
                                               )
                                         )
                                     )
                               )
                    );
            }
            else {
                $this->set('menu_left', 
                           array('active' => '0', 'sections' => array(
                                     array('name' => 'Case', 'sub' => array(
                                               array('name' => 'Cases', 'link' => '/pols')
                                               )
                                         )
                                     )
                               )
                    );
            }
        }
        
    }

    function register() {
        $this->set('username_error', 'Username must be between 6 and 40 characters.');
        if (!empty($this->data)) {
            $san = new Sanitize();
            $this->data['User']['username'] = $san->paranoid($this->data['User']['username']);
            $this->data['User']['email'] = $san->paranoid($this->data['User']['email'],
                                                               array('@', '.', '-', '+'));
            
            $this->data['User']['first_name'] = $san->paranoid($this->data['User']['first_name'],
                                                               array('\'', ' '));
            
            $this->data['User']['last_name'] = $san->paranoid($this->data['User']['last_name'],
                                                               array('\'', ' '));
            $this->User->set($this->data);
            if ($this->User->validates()) {
                if ($this->User->findByUsername($this->data['User']['username'])) {
                    $this->User->invalidate('username');
                    $this->set('username_error', 'User already exists.');
                }
                else {
                    if ($this->User->findByEmail($this->data['User']['email'])) {
                        $this->User->invalidate('email');
                    }
                    else {
                        // create group
                        $this->data['Group']['name'] = $this->data['User']['username'];
                        $this->Group->create();
                        if ($this->Group->save($this->data)) {
                            $this->data['User']['password'] = md5($this->data['User']['password']);
                            $this->data['User']['em_key'] = md5($this->data['User']['email'].$this->data['User']['password'].time());
                            $gid = $this->Group->getID();
                            $this->data['User']['group_id'] = $gid;
                            $this->User->create();
                            if ($this->User->save($this->data)) {
                                if (0) {
                                    // send email to confirm registration
                                    mail($this->data['User']['email'], "Xplico - Account Activation Request",
                                         "To confirm click your personal link below\n http://demo.xplico.org/users/registerConfirm?key=".$this->data['User']['em_key']."\n",
                                         "From: register@xplico.org");
                                    $this->Session->setFlash('To complete registration wait the email');
                                }
                                else {
                                    $this->Session->setFlash('Registration Completed');
                                    $this->User->saveField('em_checked', 1);
                                }
                                $this->redirect('/users/index');
                            }
                            else {
                                $this->Group->delete($gid);
                                $this->Session->setFlash('There was a problem saving this information');
                            }
                        }
                    }
                }
            }
            else {
                $this->validateErrors($this->User);
            }
        }
    }
    
    function registerConfirm() {
        if (!empty($this->params['url']['key'])) {
            $san = new Sanitize();
            $em_key = $this->params['url']['key'];
            $em_key = $san->paranoid($em_key);
            $results = $this->User->findByEm_key($em_key);
            if (!empty($results)) {
                $results['User']['em_checked'] = 1;
                $this->User->save($results);
            }
            else {
                $this->redirect('/users/login');
            }
        }
        else {
            $this->redirect('/users/login');
        }
    }
    
    function resend_reg() {
        // resend registration
        if (!empty($this->data)) {
            $user = $this->User->findByEmail($this->data['User']['email']);
            if (!empty($user)) {
                $em_key = $user['User']['em_key'];
                // send email to confirm registration
                mail($this->data['User']['email'], "Xplico - Account Activation Request",
                     "To confirm click your personal link below\n http://demo.xplico.org:71/users/registerConfirm?key=".$em_key."\n",
                     "From: register@xplico.org");
                $this->Session->setFlash('To complete registration wait the email');
            }
            $this->redirect('/users/index');
        }
    }

    function login() {
        if (!is_writable("/opt/xplico/xi/app/tmp/cache")) {
            $this->Session->setFlash("Error, /opt/xplico/xi/app/tmp/cache path is not writable, please fix permissions and reload");
            die(); // in this case we can view the warning.
        }

	$firstLogin = '0';	//Value for not displaying an "Invalid login" after starting Xplico.

	if (!empty($this->data['Param']['startXplico']) && ($this->data['Param']['startXplico']) == 'yesPlease') {
		//A way to start/restart Xplico from the web interface (aka "xi")
		system ("/usr/bin/sudo /usr/bin/killall -9 /opt/xplico/bin/dema > /dev/null 2>&1 &"); 
		system ("/usr/bin/sudo /opt/xplico/script/sqlite_demo.sh > /dev/null 2>&1 &");
		sleep (1);	//Necessary, starting sqlite_demo.sh takes some time and without this sleep next line would give false information.
		$firstLogin = '1';
	}

	$isXplicoRunning = $this->Xplico->checkXplicoStatus();
  	
	if ($isXplicoRunning == 0) {
                $this->Session->setFlash('Xplico is not running!<br/><br/>
		For starting Xplico, please choose <u>one</u> of these options <u>as root</u>: <br /><br />
		a) If you are using the Ubuntu/Debian package, run: "/etc/init.d/xplico start" <br />
		b) Run: "/opt/xplico/script/sqlite_demo.sh" <br />');      
        }
	else {
            $username = $this->Session->read('user');
            if ($username) {
                $this->redirect('/pols/index');
            }
            $this->set('error', false);
            if ($this->data) {
                $san = new Sanitize();
                if (!empty ($this->data['User']['username'])) {
                    $cleaned_username = $san->paranoid($this->data['User']['username']);	      
                    $results = $this->User->findByUsername($cleaned_username);
                    if ($results && $results['User']['password'] == md5($this->data['User']['password']) && $results['User']['em_checked'] == 1) {
                        $this->Session->write('userid', $results['User']['id']);
                        $this->Session->write('user', $cleaned_username);
                        $this->Session->write('last_login', $results['User']['last_login']);
                        $this->Session->write('group', $results['User']['group_id']);
                        $this->User->id = $results['User']['id'];
                        $this->User->saveField('last_login', date("Y-m-d H:i:s"));
                        $this->User->saveField('login_num', $results['User']['login_num'] + 1);
                        if ($results['User']['group_id'] == 1) {
                            // admin privilages
                            $this->Session->write('admin', 1);
                            $this->redirect('/admins');
                        }
                        else {
                            $this->redirect('/pols/index');
                        }
                    }
                    else {
                        if ($firstLogin != 1)
                            $this->Session->setFlash('Invalid login!');
                    }
                }
            }
	}
        
	$register = $this->Param->findByName('register');
	$this->set('ParamStartXplico', 'no');
        $this->set('isXplicoRunning', $isXplicoRunning);
        $this->set('register', $register['Param']['nvalue']);
    }
    
    function cpassword($id = NULL) {
        $uid = $this->Session->read('userid');
        if ($id != NULL && !$this->Session->check('admin')) {
            $this->redirect('/users/login');
            die();
        }
        else if ($id == NULL)
            $id = $uid;
        if (!empty($this->data)) {
            $id = $this->data['User']['id'];
            $usr = $this->User->read(null, $id);
            if (!empty($usr) && $usr['User']['password'] == md5($this->data['User']['opassword'])) {
                if ($this->data['User']['password'] == $this->data['User']['rpassword']) {
                    if ($this->User->saveField('password', $this->data['User']['password'], true)) {
                        // salvo in formato md5
                        $this->User->saveField('password', md5($this->data['User']['password']), true);
                        $this->Session->setFlash(__('New assword activeted!', true));
                        $this->redirect(array('action' => 'login'));
                    }
                    else {
                        $this->Session->setFlash(__('You can not change the password', true));
                    }
                }
                else {
                    $this->Session->setFlash(__('New password error', true));
                }
            }
            else {           
                $this->Session->setFlash(__('Password wrong', true));
            }
        }
        $this->set('id', $id);
    }
    
    function logout() {
        $this->Session->destroy();
        $this->redirect('/users/login');
    }
    
    function index() {
        $username = $this->Session->read('user');
        if ($username || !empty($this->data)) {
            $this->redirect('/pols/index');
        } else {
            $this->redirect('/users/login');
        }
    }
}
?>
